#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const minimist = require("minimist");
const gen_swagger_1 = require("../commands/gen-swagger");
require("../hook");
const util_1 = require("../util");
const help = `
Usage: agreed-typed [subcommand] [options]
Subcommands:
  gen-swagger                        Generate swagger file.
Options:
  --help                             Shows the usage and exits.
  --version                          Shows version number and exits.
Examples:
  agreed-typed gen-swagger --path ./agreed.ts
`.trim();
function main() {
    const argv = minimist(process.argv.slice(2), {
        stopEarly: true,
        string: ["help", "version"]
    });
    const commands = {
        ["gen-swagger"]: gen_swagger_1.generate
    };
    if (argv.help) {
        return util_1.showHelp(0, help);
    }
    if (argv.version) {
        const pack = require("../../package.json");
        process.stdout.write(pack.version);
        return;
    }
    const subcommand = argv._.shift();
    const fun = commands[subcommand];
    if (!fun) {
        return util_1.showHelp(1, help);
    }
    fun(argv._);
}
main();
//# sourceMappingURL=agreed-typed.js.map