"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path_1 = require("path");
const ts = require("typescript");
const TJS = require("typescript-json-schema");
function generateSchema(fileNames, meta) {
    const settings = { required: true, ignoreErrors: true };
    const compilerOptions = {
        noEmit: true,
        noEmitOnError: false,
        emitDecoratorMetadata: true,
        experimentalDecorators: true,
        target: ts.ScriptTarget.ES5,
        module: ts.ModuleKind.CommonJS,
        allowUnusedLabels: true
    };
    const res = p => path_1.resolve(p);
    const sources = fileNames.map(res).reduce((p, f) => {
        const src = fs.readFileSync(f).toString("utf-8");
        const sourceFile = ts.createSourceFile(f, src, ts.ScriptTarget.ES2015, true, ts.ScriptKind.TS);
        const result = ts.transform(sourceFile, [transformer]);
        p[f] = result.transformed[0];
        result.dispose();
        return p;
    }, {});
    const host = ts.createCompilerHost(compilerOptions);
    const orgSourceFile = host.getSourceFile;
    host.getSourceFile = (fileName, languageVersion, onError, shouldCreateNewSourceFile) => {
        if (sources[fileName]) {
            return sources[fileName];
        }
        return orgSourceFile(fileName, languageVersion, onError, shouldCreateNewSourceFile);
    };
    host.getSourceFileByPath = undefined;
    const program = ts.createProgram(fileNames, compilerOptions, host);
    const generator = TJS.buildGenerator(program, settings);
    return meta.map(m => {
        return Object.assign({}, m, { schema: generator.getSchemaForSymbol(m.name) });
    });
}
exports.generateSchema = generateSchema;
const transformer = (context) => (rootNode) => {
    function visit(node) {
        node = ts.visitEachChild(node, visit, context);
        if (!ts.isPropertySignature(node)) {
            return node;
        }
        const ps = node;
        if (!ts.isTypeReferenceNode(ps.type)) {
            return node;
        }
        const tr = ps.type;
        if (tr.typeName.escapedText === "Placeholder") {
            ps.type = tr.typeArguments[0];
        }
        return node;
    }
    return ts.visitNode(rootNode, visit);
};
//# sourceMappingURL=generate-schema.js.map