export declare type GET = "GET";
export declare type HEAD = "HEAD";
export declare type POST = "POST";
export declare type PUT = "PUT";
export declare type PATCH = "PATCH";
export declare type DELETE = "DELETE";
export declare type HTTPMethods = GET | HEAD | POST | PATCH | PUT | DELETE;
export declare type Capture<T extends string> = T | string;
export declare type Placeholder<T> = T | string;
export declare type Path = Array<string | Capture<string>>;
export declare type RequestBody<Method extends HTTPMethods> = Method extends POST | PATCH | PUT ? object : undefined;
export declare type Headers = object;
export declare type Query = object;
export declare type RequestDef<P extends Path, H extends Headers, Q extends Query, M extends HTTPMethods, B extends RequestBody<M>> = {
    path: P;
    method: M;
    headers?: H;
    query?: Q;
    values?: object;
    body: B;
};
export declare type Status<C extends number, T extends string> = {
    status: C;
    statusText?: T;
};
export declare type Success200 = Status<200, "OK">;
export declare type Success201 = Status<201, "Created">;
export declare type Success202 = Status<202, "Accepted">;
export declare type Success203 = Status<203, "Non-Authoritative Information">;
export declare type Success204 = Status<204, "No Content">;
export declare type Success205 = Status<205, "Reset Content">;
export declare type Success206 = Status<206, "Parial Content">;
export declare type Success207 = Status<207, "Multi-Status">;
export declare type Success208 = Status<208, "Already Reported">;
export declare type Success226 = Status<226, "IM Used">;
export declare type Redirection300 = Status<300, "Multiple Choices">;
export declare type Redirection301 = Status<301, "Moved Permanently">;
export declare type Redirection302 = Status<302, "Found">;
export declare type Redirection303 = Status<303, "See Other">;
export declare type Redirection304 = Status<304, "Not Modified">;
export declare type Redirection305 = Status<305, "Use Proxy">;
export declare type Redirection307 = Status<307, "Temporary Redirect">;
export declare type Redirection308 = Status<308, "Permanent Redirect">;
export declare type Error400 = Status<400, "Bad Request">;
export declare type Error401 = Status<401, "Unauthorized">;
export declare type Error402 = Status<402, "Payment Required">;
export declare type Error403 = Status<403, "Forbidden">;
export declare type Error404 = Status<404, "Not Found">;
export declare type Error405 = Status<405, "Method Not Allowed">;
export declare type Error406 = Status<406, "Not Acceptable">;
export declare type Error407 = Status<407, "Proxy Authentication Required">;
export declare type Error408 = Status<408, "Request Timeout">;
export declare type Error409 = Status<409, "Conflict">;
export declare type Error410 = Status<410, "Goe">;
export declare type Error411 = Status<411, "Length Required">;
export declare type Error412 = Status<412, "Precondition Failed">;
export declare type Error413 = Status<413, "Payload Too Large">;
export declare type Error414 = Status<414, "URI Too Long">;
export declare type Error415 = Status<415, "Unsupported Media Type">;
export declare type Error416 = Status<416, "Range Not Satisfiable">;
export declare type Error417 = Status<417, "Expectation Failed">;
export declare type Error418 = Status<418, "I'm teapot">;
export declare type Error421 = Status<421, "Misdirected Request">;
export declare type Error422 = Status<422, "Unprocessable Entity">;
export declare type Error423 = Status<423, "Locked">;
export declare type Error424 = Status<424, "Failed Dependency">;
export declare type Error425 = Status<425, "Too Early">;
export declare type Error426 = Status<426, "Upgrade Required">;
export declare type Error451 = Status<451, "Unavailable For Legal Reasons">;
export declare type Error500 = Status<500, "Internal Server Error">;
export declare type Error501 = Status<501, "Not Implemented">;
export declare type Error502 = Status<502, "Bad Gateway">;
export declare type Error503 = Status<503, "Service Unavailable">;
export declare type Error504 = Status<504, "Gateway Timeout">;
export declare type Error505 = Status<505, "HTTP version Not Supported">;
export declare type Error506 = Status<506, "Variant Also Negotiates">;
export declare type Error507 = Status<507, "Insufficient Storage">;
export declare type Error508 = Status<508, "Loop Detected">;
export declare type Error509 = Status<509, "Bandwidth Limit Exceeded">;
export declare type Error510 = Status<510, "Not Extended">;
export declare type Error511 = Status<511, "Network Authentication Required">;
export declare type ResponseBody = object;
export declare type ResponseDef<S extends Status<number, string>, B extends ResponseBody> = {
    body?: B;
} & S;
export declare type APIDef<M extends HTTPMethods, P extends Path, ReqHeader extends Headers, Q extends Query, ReqBody extends RequestBody<M>, RespHeader extends Headers, Resp extends ResponseDef<Status<number, string>, object>> = {
    request: RequestDef<P, ReqHeader, Q, M, ReqBody>;
    response: {
        headers?: RespHeader;
        values?: object;
    } & Resp;
};
export declare function convert(...apis: Array<{
    request: any;
}>): {
    request: any;
}[];
